import snowflake.connector


class SnowflakeConnection:
    _dbconn = None
    _config = None

    def open(self, config):
        self._config = config

        self.close()

        host = config["HOST"]
        user = config["USERNAME"]
        password = config["PASSWORD"]
        account = config["ACCOUNT_ID"]
        warehouse = config["WAREHOUSE"]
        role = config["ROLE_NAME"]

        self._dbconn = snowflake.connector.connect(
            host=host, user=user, password=password, account=account, warehouse=warehouse)

        self.executeNonQuery("USE ROLE {}".format(role))
        self.executeNonQuery("USE WAREHOUSE {}".format(warehouse))

    def close(self):
        if self._dbconn is not None:
            try:
                self._dbconn.close()
            except Exception:
                pass

            self._dbconn = None

    def executeNonQuery(self, sql):
        d = self._dbconn.cursor()

        rc = d.execute(sql)

        rowsAffected = rc.rowcount

        d.close()

        return rowsAffected

    def executeMany(self, sql, rows):
        d = self._dbconn.cursor()

        rc = d.executemany(sql, rows)

        rowsAffected = rc.rowcount

        d.close()

        return rowsAffected

    def fetchRow(self, sql):
        d = self._dbconn.cursor()

        try:
            d.execute(sql)

            row = d.fetchone()

            d.close()
        except Exception as e:
            d.close()
            raise e

        return row

    def fetchRows(self, sql):
        d = self._dbconn.cursor()

        try:
            d.execute(sql)

            rows = d.fetchall()

            d.close()
        except Exception as e:
            d.close()
            raise e

        return rows

    def fetchRowsDF(self, sql):
        c = self._dbconn.cursor()

        try:
            cur = c.execute(sql)

            rows = cur.fetch_pandas_all()

            return rows
        except Exception as e:
            raise e

    def rollback(self):
        self._dbconn.rollback()

    def commit(self):
        self._dbconn.commit()

    def getDMLRowsAffected(self, rowsAffected):
        try:
            sql = "select * from table(RESULT_SCAN ( LAST_QUERY_ID() ))"

            row = self.fetchRow(sql)

            return row
        except Exception:
            return None

    def getMultipleDMLRowsAffected(self, rowsAffected):
        try:
            sql = "select * from table(RESULT_SCAN ( LAST_QUERY_ID() ))"

            rows = self.fetchRow(sql)

            return rows
        except Exception:
            return [{"inserted": 0, "updated": rowsAffected}]



import sys
import os
import getopt
import configparser
import logging
import json

from datetime import datetime
from Amazon import S3Client

import SMTPMailer 
import Amazon
from sys import stdout


def read_file(filename):
	file = open(filename, "r")

	str = file.read()

	file.close()
 
	return str

#########################################################################
#
# get_configuration:
#
# This function will return the configuration stored in the 
# configuration file "config.ini" located in the same directory
# as the script
#
# Returns:
#
# 	Instance of a two dimension array containing the configuration 
# 	values with first dimension the section and the second dimension 
# 	is the key
#
#########################################################################
def get_configuration(filename):
	config = configparser.ConfigParser()

	config.read(filename)

	return config


############################################################################
#
# get_environment:
#
# Retrieves the environment from configuration file
#
# Arguments:
#
#   None
#
# Returns:
#
# 	Name of the environment
#
############################################################################
def get_environment(config, appsection):
	configenv = config[appsection]["environment"].lower()

	if configenv == "dev" or configenv == "staging" or configenv == "sit" or configenv == "prod" or configenv == "prodread":
		environ = configenv
	else:
		raise Exception("Invalid configuration value [{}] for environment. Valid values are: 'dev', 'staging', 'sit', 'prod', or 'prodread'".format(configenv))

	return environ

def get_db_environment(config, appsection):
	configenv = config[appsection]["dbenvironment"].lower()

	if configenv == "dev" or configenv == "staging" or configenv == "sit" or configenv == "prod" or configenv == "prodread":
		environ = configenv
	else:
		raise Exception("Invalid configuration value [{}] for dbenvironment. Valid values are: 'dev', 'staging', 'sit', 'prod', or 'prodread'".format(configenv))

	return environ

def get_s3_environment(config, appsection):
	configenv = config[appsection]["s3environment"].lower()

	if configenv == "dev" or configenv == "staging" or configenv == "prod":
		environ = configenv
	else:
		raise Exception("Invalid configuration value [{}] for s3environment. Valid values are: 'dev', 'staging', or 'prod'".format(configenv))

	return environ

def get_snowflake_environment(config, appsection):
	configenv = config[appsection]["snowflakeenvironment"].lower()

	if configenv == "dev" or configenv == "staging" or configenv == "prod":
		environ = configenv
	else:
		raise Exception("Invalid configuration value [{}] for snowflakeenvironment. Valid values are: 'dev', 'staging', or 'prod'".format(configenv))

	return environ

def get_commandline_arguments(longargs):

	myopts, args = getopt.getopt(sys.argv[1:], '', longargs)

	args = {}

	for k,v in myopts:
		args.update({ k.replace('--', '') : v})

	return args
      
def get_default_config_filename():
	return os.path.dirname(os.path.realpath(__file__)) + os.path.sep + "config" + os.path.sep + "config.ini"		

def get_default_config_path():
	return os.path.dirname(os.path.realpath(__file__)) + os.path.sep + "config" + os.path.sep	


############################################################################
#
# get_config_filename:
#
# Retrieves the configuration filename from the command line arguments
#
# Arguments:
#
#   None
#
# Returns:
#
# 	Configuration filename
#
############################################################################
def get_config_filename():
	default_cfilename = get_default_config_filename()		
 
	if len(sys.argv) == 1:
		cfilename = default_cfilename
	else:
		cfilename = sys.argv[1]

	return cfilename

"""
def send_email(mailHost, mailUsername, mailPassword, mailPort, mailFrom, mailTo, mailCC, mailSubject, msg):
	mail = MAIL.SMTPMailer()
 
	mail.send(mailHost, mailUsername, mailPassword, mailPort, mailFrom, mailTo, mailCC, mailSubject, msg)
"""

def get_dbconfig(config, env):
	return config[env + "-rds-database"]

def create_logger(logdir, logginglevel, fileprefix):
	d = datetime.today()

	logfilename = "{}{}_{}{}{}.log".format(logdir, fileprefix, d.year, str(d.month).rjust(2, '0'), str(d.day).rjust(2, '0'))

	logging.basicConfig(filename=logfilename, level=logginglevel, format='%(asctime)s.%(msecs)03d %(levelname)s %(module)s - %(funcName)s: %(message)s',
															datefmt='%Y-%m-%d %H:%M:%S' )

def get_log_directory(config, scriptSectionName):
	logdirectory = config[scriptSectionName]["logdir"]

	if (logdirectory.endswith(os.path.sep) == False):
		logdirectory = logdirectory + os.path.sep

	return logdirectory
    
def get_logging_level(config, scriptSectionName):
	if config[scriptSectionName]["loglevel"].lower() == "debug":
		loglevel = logging.DEBUG
	elif config[scriptSectionName]["loglevel"].lower() == "info":
		loglevel = logging.INFO
	elif config[scriptSectionName]["loglevel"].lower() == "warning":
		loglevel = logging.WARNING
	else:
		loglevel = logging.ERROR

	return loglevel

    
def getJsonConfig(fileName):
	with open(fileName)  as f:
		jsonConfig = json.load(f)
  
	return jsonConfig

def getS3Configuration(config, env):
	return config[env + "-s3"]

def getBaseFilename(filename, separator = "/"):
	parts = filename.split(separator)
	return parts[len(parts) - 1]

def getCommandLineArguments(shortOpts, longOpts):
	try:
		opts, args = getopt.getopt(sys.argv[1:], shortOpts, longOpts)
	except getopt.GetoptError as err:
		print(str(err))
		return None

	args = {}

	for o, a in opts:
		args[o] = a

	return args

def pandas_to_csv_file(df, filename):
	csvdata = df.to_csv(index=False, line_terminator="\n")

	try:
		fh = open(filename, "w")
		fh.write(csvdata)
		fh.close()
	except Exception as e:
		print("Error writing to file {}: {}".format(filename, e))
		sys.exit(2)

def set_library_log_level(libname, loggingLevel):
	root = logging.getLogger(libname)
	root.setLevel(loggingLevel)
#	handler = logging.StreamHandler(sys.stdout)
#	handler.setLevel(loggingLevel)
#	formatter = logging.Formatter('%(asctime)s %(levelname)s %(module)s:%(lineno)s - %(message)s')
#	handler.setFormatter(formatter)
#	root.addHandler(handler)

def init_logging_to_screen(loggingLevel):
	root = logging.getLogger()
	root.setLevel(loggingLevel)

	handler = logging.StreamHandler(sys.stdout)
	handler.setLevel(loggingLevel)
	formatter = logging.Formatter('%(asctime)s %(levelname)s %(module)s:%(lineno)s - %(message)s')
	handler.setFormatter(formatter)
	root.addHandler(handler)

def get_filenames_to_process(s3Client, appConfig, interactive=False):
    try:
        filenames = s3Client.getFilenames(appConfig["s3"]["source_bucket"],
                                            appConfig["s3"]["source_incoming_path"],
                                            appConfig["s3"]["source_file_prefix"], None)
    except Exception as e:
        raise Exception(f"Error retriving S3 files names: {e}")

    if len(filenames) == 0:
        return []

    fileNamesToProcess = []

    if len(filenames) > 1 and interactive:
        choices = []

        print("")
        print("Possible files to process:\n")

        maxLen = 0

        for file in filenames:
            f = file['filename'].split("/")
            fname = f[len(f) - 1]
            if len(fname) > maxLen:
                maxLen = len(fname)

        print("{}  {}     File Date".format(" ".rjust(len(str(len(filenames)))),
                                            "Filename".ljust(maxLen)))
        print("{}  {}     ===================".format("=".rjust(len(str(len(filenames)))),
                                                        "=".ljust(maxLen, "=")))

    #        print("  " + "=".ljust(maxLen, "=") + "     ===================")

        choiceNum = 1

        for file in filenames:
            f = file['filename'].split("/")
            fname = f[len(f) - 1]
            choices.append(fname)
            print("{}  {}     {}".format(str(choiceNum).rjust(len(str(len(filenames)))),
                                            fname.ljust(maxLen),
                                            file['moddate'].strftime("%Y-%m-%d %H:%M:%S")))
            choiceNum = choiceNum + 1

        while True:
            c = input("\nEnter choice (1 to {}  0 to Quit): ".format(len(filenames)))
            if c.isnumeric() and int(c) >= 0 and int(c) <= len(filenames):
                break
        if int(c) == 0:
            os._exit(0)

        fileToProcess = choices[int(c) - 1]

        fileNamesToProcess.append(fileToProcess)
    else:
        for i, filename in enumerate(filenames):
            f = filenames[i]['filename'].split("/")
            fileNamesToProcess.append(f[len(f) - 1])

    return fileNamesToProcess

def get_input_from_user(prompt, required, possibleValues=None, convertToLower=True):
    while True:
        if not possibleValues is None:
            s = ""

            for str in possibleValues:
                if s != "":
                    s += ","
                s += str
    
            promptText = "{} (" + s + "): "
        else:
            promptText = "{} : "

        c = input(promptText.format(prompt)).strip()

        if convertToLower:
            c = c.lower()

        if required:
            if c == "":
                continue
        if possibleValues is not None:
            if c not in possibleValues:
                continue

        break

    return c


def get_boolean_input_from_user(prompt):
    while True:
        c = input("{}: ".format(prompt)).strip().lower()

        if c == "y" or c == "n":
            break

    if c == "y":
        return True

    return False


def get_int_input_from_user(prompt, required, defaultValue):
    while True:
        c = input("{}: ".format(prompt)).strip().lower()

        if required and c == "":
            c = int(defaultValue)
        else:
            if not c.isnumeric():
                continue

            c = int(c)

        break

    return c

def get_date_yyyymmddhhmmss():
    return datetime.now().strftime("%Y%m%d%H%M%S")

def log(msg, msgtype="INFO"):
    print(f"{datetime.now().strftime('%Y-%m-%d %H:%M:%S')}  {msgtype}  {msg}")
    stdout.flush()
    





import os
from datetime import datetime
import time
import logging
import json
import boto3
from botocore.exceptions import ClientError


#########################################################################
#
# Class S3Client:
#
# This class manages connections to an Amazon S3 server
#
#########################################################################


class S3Client:
    _accessKeyId = ""
    _secretKey = ""
    _client = None

    def __init__(self, accessKeyId, secretKey):
        self._accessKeyId = accessKeyId
        self._secretKey = secretKey

        logger = logging.getLogger("botocore")
        logger.setLevel(logging.ERROR)

        self._client = boto3.client(
            "s3",
            aws_access_key_id=self._accessKeyId,
            aws_secret_access_key=self._secretKey,
        )

    def fileExists(self, bucketName, s3Dir, fileName):
        fileNames = self.getFilenames(bucketName, s3Dir, fileName)

        if len(fileNames) > 0:
            return True

        return False

    def deleteFiles(self, bucketName, s3Dir, filePrefix):
        fileNames = self.getFilenames(bucketName, s3Dir, filePrefix)

        for fileName in fileNames:
            self._client.delete_object(Bucket=bucketName, Key=fileName)

    def uploadFile(self, sourceFile, bucketName, s3Dir, s3BaseFilename):
        s3_client = boto3.client(
            "s3",
            aws_access_key_id=self._accessKeyId,
            aws_secret_access_key=self._secretKey,
        )

        key = s3Dir

        if s3Dir.endswith("/") is False:
            key = key + "/"

        key = key + s3BaseFilename

        s3_client.upload_file(sourceFile, bucketName, key)

    def copy_file_to_bucket(self, sourceBucket, sourceKey, destBucket, destKey):
        s3 = boto3.resource(
            "s3",
            aws_access_key_id=self._accessKeyId,
            aws_secret_access_key=self._secretKey,
        )

        copy_source = {"Bucket": sourceBucket, "Key": sourceKey}
        bucket = s3.Bucket(destBucket)
        bucket.copy(copy_source, destKey)

    def getFilenames(self, bucketName, dir, prefix, extension=None):
        filePrefix = dir

        if filePrefix.endswith("/") is False:
            filePrefix = filePrefix + "/"

        filePrefix = filePrefix + prefix

        print(f"Checking s3://{bucketName}/{filePrefix} for files....")

        objs = self._client.list_objects_v2(Bucket=bucketName, Prefix=filePrefix)

        fileKeys = []

        if "Contents" not in objs.keys():
            return fileKeys

        for obj in objs["Contents"]:
            if obj["Key"][-1:] != "/":
                if extension is not None and obj["Key"].endswith(extension) is False:
                    continue
                fileKeys.append({"filename": obj["Key"], "moddate": obj["LastModified"]})

        return fileKeys

    def readFile(self, s3Bucket, key, format="text"):
        obj = self._client.get_object(Bucket=s3Bucket, Key=key)
        file_content = obj["Body"].read().decode("utf-8")
        if format == "json":
            contents = json.loads(file_content)
        else:
            contents = file_content
        return contents

    def getDirectories(self, bucketName, dir, prefix):
        filePrefix = dir

        if filePrefix.endswith("/") is False:
            filePrefix = filePrefix + "/"

        filePrefix = filePrefix + prefix

        objs = self._client.list_objects_v2(Bucket=bucketName, Prefix=filePrefix)

        fileKeys = []

        if "Contents" not in objs.keys():
            return fileKeys

        for obj in objs["Contents"]:
            if obj["Key"][-1:] == "/":
                continue

            fileKeys.append(obj["Key"])

        return fileKeys

    def download(self, bucketName, fileKey, outputDir, extension=""):
        outputFilename = outputDir
        if outputFilename.endswith(os.path.sep) is False:
            outputFilename = outputFilename + os.path.sep
        fileParts = fileKey.split("/")
        outputFilename = outputFilename + fileParts[len(fileParts) - 1]
        if extension != "" and not outputFilename.endswith(extension):
            outputFilename += extension

        self._client.download_file(bucketName, fileKey, outputFilename)

        return outputFilename

    def move(self, sourceBucketName, sourceFullFileKey, destFullFileKey):
        copy_source = {"Bucket": sourceBucketName, "Key": sourceFullFileKey}

        self._client.copy(copy_source, sourceBucketName, destFullFileKey)
        self._client.delete_object(Bucket=sourceBucketName, Key=sourceFullFileKey)

    def createDirectory(self, bucketName, s3Dir):
        newDir = s3Dir

        if newDir.endswith("/") is False:
            newDir = newDir + "/"

        self._client.put_object(Bucket=bucketName, Key=newDir)


class SecretManager:
    """AWS Secrets Manager class"""

    @staticmethod
    def get_secret(secretName: str, awsAccessKeyId: str, awsSecretKey: str, regionName: str):
        """Get the Secret from AWS"""
        """Attributes:"""
        secret_name = secretName
        region_name = regionName

        # Create a Secrets Manager client
        session = boto3.session.Session(
            aws_access_key_id=awsAccessKeyId,
            aws_secret_access_key=awsSecretKey,
            region_name=regionName,
        )
        client = session.client(service_name="secretsmanager", region_name=region_name)

        secret = []

        try:
            get_secret_value_response = client.get_secret_value(SecretId=secret_name)
            # Decrypts secret using the associated KMS CMK.
            # Depending on whether the secret is a string or binary, one of these fields will be populated.
            if "SecretString" in get_secret_value_response:
                secret = json.loads(get_secret_value_response["SecretString"])
            else:
                pass
        except ClientError as e:
            raise e

        return secret


class ParameterStore:
    @staticmethod
    def get_parameter_store_value(key, withDecryption=False):
        ssm = boto3.client("ssm", region_name="us-east-1")

        parameter = ssm.get_parameter(Name=key, WithDecryption=withDecryption)

        if parameter is None or "Parameter" not in parameter.keys():
            return ""

        return parameter["Parameter"]["Value"]


class CloudWatch:
    @staticmethod
    def format_dimension(dimensionName, dimensionValue):
        return {"Name": dimensionName, "Value": dimensionValue}

    @staticmethod
    def write_metric_data(regionName, metricNamespace, dimensions, metricName, metricUnit, metricValue):
        cloudwatch = boto3.client("cloudwatch", region_name=regionName)

        cloudwatch.put_metric_data(
            MetricData=[
                {
                    "MetricName": metricName,
                    "Dimensions": dimensions,
                    "Unit": metricUnit,
                    "Value": metricValue,
                },
            ],
            Namespace=metricNamespace,
        )


class CloudWatchLogs:
    def write_log(self, logGroupName, regionName, msg, sequenceToken=None):
        client = boto3.client("logs", region_name=regionName)

        logStreamName = "log-stream-{}".format(datetime.today().strftime("%Y%m%d"))

        try:
            response = client.create_log_stream(logGroupName=logGroupName, logStreamName=logStreamName)
        except Exception:
            pass

        currentTime = int(round(time.time() * 1000))

        if sequenceToken is None:
            try:
                response = client.describe_log_streams(
                    logGroupName=logGroupName,
                    logStreamNamePrefix=logStreamName,
                    orderBy="LogStreamName",
                    descending=True,
                    limit=1,
                )
            except Exception as e:
                logging.error(e)

            if "uploadSequenceToken" in response["logStreams"][0].keys():
                sequenceToken = response["logStreams"][0]["uploadSequenceToken"]

        try:
            if sequenceToken is not None:
                response = client.put_log_events(
                    logGroupName=logGroupName,
                    logStreamName=logStreamName,
                    logEvents=[
                        {"timestamp": currentTime, "message": msg},
                    ],
                    sequenceToken=sequenceToken,
                )
            else:
                response = client.put_log_events(
                    logGroupName=logGroupName,
                    logStreamName=logStreamName,
                    logEvents=[
                        {"timestamp": currentTime, "message": msg},
                    ],
                )
        except (
            client.exceptions.InvalidSequenceTokenException,
            client.exceptions.DataAlreadyAcceptedException,
        ) as e:
            msg = str(e)
            logging.info("Log exception: {}".format(msg))
            parts = msg.split(":")
            sequenceToken = parts[len(parts) - 1].strip()
            logging.info("Log next token: {}".format(sequenceToken))
            self.write_log(logGroupName, msg, sequenceToken)


class AWSLogger:
    _regionName = "us-east-1"
    _logGroupName = ""

    def __init__(self, logGroupName, regionName):
        self._logGroupName = logGroupName
        self._regionName = regionName

    def log(self, loggingLevel, appLevel, msg):
        outputMsg = ""

        if appLevel < 1:
            outputMsg = msg
        else:
            outputMsg = "{}>{}".format("".ljust((appLevel) * 4, "-"), msg)

        if loggingLevel == logging.DEBUG:
            logging.debug(outputMsg)
        elif loggingLevel == logging.INFO:
            logging.info(outputMsg)
        elif loggingLevel == logging.WARNING:
            logging.warning(outputMsg)
        elif loggingLevel == logging.ERROR:
            logging.error(outputMsg)
        else:
            logging.error(outputMsg)

        if loggingLevel == logging.DEBUG:
            return

        cloudWatchLogs = CloudWatchLogs()

        cloudWatchLogs.write_log(self._logGroupName, self._regionName, msg)



# import mysql.connector
import pymysql
import time
from sshtunnel import SSHTunnelForwarder

class OtfMySQLConnection():
    _config = None
    _dbsection = ""
    _sshTunnel = None
    _dbconn = None
    _otbaseSchemaName = "OTbase"
    _otbaseStatSchemaName = "OTbaseStat"
    _otpushSchemaName = "OTpush"
    _otstageSchemaName = "OTstage"
    _otstageETLSchemaName = "OTstageETL"
    _otstageOTInsightsSchemaName = "OTstageOTInsights"
    _otcrmSchemaName = "OTcrm"
    _currentCursor = None

    def __init__(self, dbSchemas):
        self._otbaseSchemaName = dbSchemas['OTbase']
        self._otpushSchemaName = dbSchemas['OTpush']
        self._otstageSchemaName = dbSchemas['OTstage']
        self._otstageETLSchemaName = dbSchemas['OTstageETL']
        self._otstageOTInsightsSchemaName = dbSchemas['OTstageOTInsights']
        self._otbaseStatSchemaName = dbSchemas['OTbaseStat']
        self._otcrmSchemaName = dbSchemas['OTcrm']

    def open(self, host, user, password, dbName, port=3306):
        #        self._dbconn = mysql.connector.connect(
        #            host=host, user=user, passwd=password, port=port, database=dbName)
        self._dbconn = pymysql.connect(
            host=host, user=user, passwd=password, port=int(port), database=dbName)

    def open_ssh_tunnel(self, host, user, password, dbName, port=3306):
        self.close()

        try:
            self._sshTunnel = self.create_ssh_tunnel(host, port)
            self._sshTunnel.start()
        except Exception as ex:
            self._sshTunnel = None
            raise ex

        port = self._sshTunnel.local_bind_port
        hostname = "127.0.0.1"

        x = range(6)

        for n in x:
            try:
                self._dbconn = None

                self._dbconn = pymysql.connect(user=user,
                                               passwd=password,
                                               host=hostname,
                                               database=dbName,
                                               port=port,
                                               charset='utf8',
                                               use_unicode=True)

                break
            except Exception as e:
                self._dbconn = None
                print("Error connecting to database: {}".format(str(e)))
                time.sleep(5)

        if self._dbconn is None:
            self.close()
            raise Exception("Unable to connect to database.  Retries have been exceeded")

        print("Connected to RDS host {} port {}".format(host, port))

        try:
            self.executeNonQuery("SET SESSION TRANSACTION ISOLATION LEVEL READ COMMITTED")

            row = self.fetchRow("SELECT @@SESSION.tx_isolation as isolevel")

            if row:
                print("Isolation level is now {}".format(row["isolevel"]))
            else:
                print("Isolation level is unknown")

        except Exception as e:
            print("Error setting transaction isolation level", e)

        return port

    def close(self):
        if self._dbconn is not None:
            try:
                self._dbconn.close()
            except Exception:
                pass

            self._dbconn = None

    def create_ssh_tunnel(self, host, port):
        sshkey = "/Users/rroyce/Documents/sslkeys/otf-bastion-prod.pem"
        jumpserver = "prodbastion.orangetheory.co"
        sshuser = "ec2-user"
        sshlocalport = 5432
        sshport = 22

        tunnel = None
        tries_ = 0

        if tunnel is None and tries_ < 5:
            try:
                print(f"Connecting to sshtunnel [{jumpserver}] ssh port [{sshport}]  ssh username [{sshuser}]  ssh key [{sshkey}]  remote host [{host}]  remote port [{port}]")
                tunnel = SSHTunnelForwarder(
                    (jumpserver, sshport),
                    ssh_username=sshuser,
                    ssh_pkey=sshkey,
                    remote_bind_address=(
                        host,
                        int(port)
                    )
                )
            finally:
                tries_ += 1
                time.sleep(.5)

        return tunnel

    def fetchRow(self, sql):
        d = self._dbconn.cursor(pymysql.cursors.DictCursor)

        d.execute(sql)

        row = d.fetchone()

        d.close()

        return row

    def fetchRows(self, sql, asDict=True):
        if asDict:
            d = self._dbconn.cursor(pymysql.cursors.DictCursor)
        else:
            d = self._dbconn.cursor()

        d.execute(sql)

        rows = d.fetchall()

        d.close()

        return rows

    def fetchRowsMany(self, sql, blocksize, closecursor):
        if self._currentCursor is None:
            self._currentCursor = self._dbconn.cursor(
                pymysql.cursors.DictCursor)
            self._currentCursor.execute(sql)

        rows = self._currentCursor.fetchmany(int(blocksize))

        if rows is None or closecursor:
            self._currentCursor.close()
            self._currentCursor = None

        return rows

    def executeNonQuery(self, sql):
        d = self._dbconn.cursor(pymysql.cursors.DictCursor)

        rowsAffected = d.execute(sql)

        d.close()

        return rowsAffected

    def executemany(self, sql, data):
        d = self._dbconn.cursor(pymysql.cursors.DictCursor)

        rowsAffected = d.executemany(sql, data)

        d.close()

        return rowsAffected

    def truncateTable(self, tableName):
        cur = self._dbconn.cursor()

        cur.execute("TRUNCATE TABLE {}".format(tableName))

        cur.close()

    def dropTable(self, tableName):
        sql = "DROP TABLE IF EXISTS {}".format(tableName)

        self.executeNonQuery(sql)

    def tableExists(self, tableSchema, tableName):
        sql = "SELECT table_name FROM information_schema.tables where table_schema = '{}' and table_name = '{}'".format(
            tableSchema, tableName)

        row = self.fetchRow(sql)

        return row is not None

    def OTbaseSchema(self):
        return self._otbaseSchemaName

    def OTbaseStatSchema(self):
        return self._otbaseStatSchemaName

    def OTpushSchema(self):
        return self._otpushSchemaName

    def OTcrmSchema(self):
        return self._otcrmSchemaName

    def OTstageSchema(self):
        return self._otstageSchemaName

    def OTstageETLSchema(self):
        return self._otstageETLSchemaName

    def rollback(self):
        self._dbconn.rollback()

    def commit(self):
        self._dbconn.commit()

    def get_connection(self):
        return self._dbconn

    def callStoredProcedureWithRead(self, storedProcName, args):
        d = self._dbconn.cursor(pymysql.cursors.DictCursor)

        d.callproc(storedProcName, args)

        rows = d._rows

        d.close()

        return rows



import pathlib
from setuptools import setup, find_packages

# The directory containing this file
HERE = pathlib.Path(__file__).parent

# The text of the README file
README = (HERE / "README.md").read_text()

#     packages=["otfcommon"], """


# This call to setup() does all the work
setup(
    name="otfpythonpkg",
    version="0.2.0",
    description="Provides common classes that will be shared between python applications",
    long_description=README,
    long_description_content_type="text/markdown",
    url="https://github.com/ronroyce22/otfpythonpkg.git",
    author="Ron Royce",
    author_email="rroycems@outlook.com",
    license="MIT",
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.9",
    ],
    packages=find_packages(exclude=("tests",)),
    include_package_data=True,
    install_requires=["feedparser", "html2text"],
    entry_points={
    },
)
